"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CoreService = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const firewall_1 = require("./firewall");
const sequelize_storage_1 = require("./storages/sequelize.storage");
const multiple_storage_1 = require("./storages/multiple.storage");
const hardcoded_memory_storage_1 = require("./storages/hardcoded-memory.storage");
const iacry_error_1 = require("./errors/iacry.error");
const matcher_1 = require("./matcher");
const constants_1 = require("./constants");
const policy_1 = require("./interfaces/policy");
const core_1 = require("./helpers/core");
const entity_1 = require("./decorators/entity");
const ioredis_1 = require("./storages/cache/ioredis");
const cached_storage_1 = require("./storages/cached.storage");
let CoreService = class CoreService extends core_1.CoreHelper {
    constructor(options) {
        super();
        this.options = options;
        this.storage = new multiple_storage_1.MultipleStorage();
        if (options.storage) {
            switch (typeof options.storage) {
                case 'string':
                    switch (options.storage.toLowerCase()) {
                        case constants_1.SEQUELIZE_STORAGE:
                            this.storage.storages.push(new sequelize_storage_1.SequelizeStorage(options.storageRepository));
                            break;
                        default:
                            throw new iacry_error_1.BaseError(`Unrecognized PolicyInterface Storage type: ${options.storage}`);
                    }
                    break;
                default:
                    this.storage.storages.push(options.storage);
            }
        }
        if (options.policies) {
            this.storage.storages.push(new hardcoded_memory_storage_1.HardcodedMemoryStorage(options.policies));
        }
        if (options.cache) {
            let cache;
            switch (typeof options.cache) {
                case 'string':
                    switch (options.cache.toLowerCase()) {
                        case constants_1.IOREDIS_CACHE:
                            cache = new ioredis_1.IoRedis(options.cacheClient);
                            break;
                        default:
                            throw new iacry_error_1.BaseError(`Unrecognized PolicyCache type: ${options.cache}`);
                    }
                    break;
                default:
                    cache = options.cache;
            }
            this.storage = new cached_storage_1.CachedStorage(this.storage, cache, options.cacheOptions);
        }
        this.firewall = firewall_1.Firewall.create(this.storage, new matcher_1.Matcher(options.strict));
    }
    attach(rawPrincipal, rawPolicies) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const principal = entity_1.isEntity(rawPrincipal)
                ? entity_1.toDynamicIdentifier(rawPrincipal)
                : (this.normalizeDynamicIdentifier(rawPrincipal, policy_1.PRINCIPAL));
            return this.storage.add(principal, rawPolicies);
        });
    }
    reset(rawPrincipal, rawPolicies) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const principal = entity_1.isEntity(rawPrincipal)
                ? entity_1.toDynamicIdentifier(rawPrincipal)
                : (this.normalizeDynamicIdentifier(rawPrincipal, policy_1.PRINCIPAL));
            return rawPolicies
                ? this.storage.save(principal, rawPolicies)
                : this.storage.purge(principal);
        });
    }
    retrieve(rawPrincipal) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const principal = entity_1.isEntity(rawPrincipal)
                ? entity_1.toDynamicIdentifier(rawPrincipal)
                : (this.normalizeDynamicIdentifier(rawPrincipal, policy_1.PRINCIPAL));
            const rawPolicies = yield this.storage.fetch(principal);
            return rawPolicies.map((rawPolicy) => typeof rawPolicy === 'string'
                ? JSON.parse(rawPolicy)
                : rawPolicy);
        });
    }
    grant(rawAction, rawPrincipal, rawResource = core_1.CoreHelper.ANY, effect = policy_1.Effect.ALLOW, sid) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const resource = entity_1.isEntity(rawResource)
                ? entity_1.toDynamicIdentifier(rawResource)
                : rawResource;
            const principal = entity_1.isEntity(rawPrincipal)
                ? entity_1.toDynamicIdentifier(rawPrincipal)
                : rawPrincipal;
            return this.attach(rawPrincipal, [
                {
                    [policy_1.SID]: sid,
                    [policy_1.EFFECT]: effect,
                    [policy_1.ACTION]: rawAction,
                    [policy_1.RESOURCE]: resource,
                    [policy_1.PRINCIPAL]: principal,
                },
            ]);
        });
    }
    isGranted(rawAction, rawPrincipal, rawResource = core_1.CoreHelper.ANY, rule = constants_1.IS_ALLOWED) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            switch (rule) {
                case constants_1.IS_ALLOWED:
                case constants_1.IS_ALLOWED_ANY:
                case constants_1.IS_ALLOWED_IMPLICIT:
                    break;
                default:
                    throw new iacry_error_1.BaseError(`Unrecognized firewall rule: ${rule}`);
            }
            const resource = entity_1.isEntity(rawResource)
                ? entity_1.toDynamicIdentifier(rawResource)
                : rawResource;
            const principal = entity_1.isEntity(rawPrincipal)
                ? entity_1.toDynamicIdentifier(rawPrincipal)
                : rawPrincipal;
            return this.firewall[rule](resource, rawAction, principal);
        });
    }
};
CoreService = tslib_1.__decorate([
    common_1.Injectable(),
    tslib_1.__param(0, common_1.Inject(constants_1.IACRY_OPTIONS)),
    tslib_1.__metadata("design:paramtypes", [Object])
], CoreService);
exports.CoreService = CoreService;
