"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Matcher = void 0;
const policy_1 = require("./interfaces/policy");
const policy_2 = require("./policy");
const core_1 = require("./helpers/core");
class Matcher extends core_1.CoreHelper {
    constructor(strict = false) {
        super();
        this.strict = strict;
        this.NEGATION_REGEXP = new RegExp(`^\s*${policy_1.NEGATION}`);
    }
    match(rawResource, rawAction, rawPrincipal, policies) {
        const resource = this.normalizeDynamicIdentifier(rawResource, policy_1.RESOURCE);
        const action = this.normalizeDynamicIdentifier(rawAction, policy_1.ACTION);
        const principal = this.normalizeDynamicIdentifier(rawPrincipal, policy_1.PRINCIPAL);
        let result = {
            allow: [],
            deny: [],
            abstain: [],
        };
        for (const policy of policies) {
            const resourceMatch = this.matchDynamicIdentifierVector(resource, policy.property(policy_1.RESOURCE), policy_1.RESOURCE);
            const actionMatch = this.matchDynamicIdentifierVector(action, policy.property(policy_1.ACTION), policy_1.ACTION);
            const principalMatch = this.matchDynamicIdentifierVector(principal, policy.property(policy_1.PRINCIPAL), policy_1.PRINCIPAL);
            const match = resourceMatch && actionMatch && principalMatch;
            if (match && policy.Effect === policy_1.Effect.ALLOW) {
                result.allow.push(policy);
            }
            else if (match && policy.Effect === policy_1.Effect.DENY) {
                result.deny.push(policy);
            }
            else {
                result.abstain.push(policy);
            }
        }
        return result;
    }
    matchDynamicIdentifierVector(source, targets, prop) {
        return (targets
            .map((target) => this.matchDynamicIdentifier(source, target, prop))
            .filter(Boolean).length === targets.length);
    }
    matchDynamicIdentifier(source, target, prop) {
        const matchProps = policy_1.DYNAMIC_IDENTIFIER_PROPS_MAPPING[prop];
        return (matchProps
            .map((key) => this.matchItem(source[key], target[key]))
            .filter(Boolean).length === matchProps.length);
    }
    matchItem(rawSource, rawTarget) {
        const sourceVector = this.normalize(rawSource);
        const targetVector = this.normalize(rawTarget);
        for (const source of sourceVector) {
            for (const target of targetVector) {
                if (source === policy_2.Policy.ANY ||
                    target === policy_2.Policy.ANY ||
                    this.matchSingle(source, target)) {
                    return true;
                }
            }
        }
        return false;
    }
    matchSingle(source, target) {
        if (!this.isNegated(source) && !this.isNegated(target)) {
            return source === target;
        }
        else if (this.isNegated(source) && this.isNegated(target)) {
            return this.extractNegated(source) === this.extractNegated(target);
        }
        else if (this.isNegated(source)) {
            return this.extractNegated(source) !== target;
        }
        else {
            return source !== this.extractNegated(target);
        }
    }
    normalize(rawValue) {
        let value = rawValue.toString();
        return (this.strict ? value : value.toLowerCase())
            .split(policy_1.OR)
            .map((x) => x.trim())
            .filter(Boolean);
    }
    extractNegated(rawValue) {
        return rawValue.replace(this.NEGATION_REGEXP, '');
    }
    isNegated(rawValue) {
        return this.NEGATION_REGEXP.test(rawValue);
    }
}
exports.Matcher = Matcher;
